<?php

/**
 * This file is part of the Phalcon Framework.
 *
 * (c) Phalcon Team <team@phalcon.io>
 *
 * For the full copyright and license information, please view the LICENSE.txt
 * file that was distributed with this source code.
 */

declare(strict_types=1);

namespace Phalcon\Tests\Integration\Storage\Adapter;

use Codeception\Example;
use IntegrationTester;
use Memcached as NativeMemcached;
use Phalcon\Storage\Adapter\Apcu;
use Phalcon\Storage\Adapter\Libmemcached;
use Phalcon\Storage\Adapter\Memory;
use Phalcon\Storage\Adapter\Redis;
use Phalcon\Storage\Adapter\Stream;
use Phalcon\Storage\Adapter\Weak;
use Phalcon\Storage\SerializerFactory;
use Redis as NativeRedis;

use function getOptionsLibmemcached;
use function getOptionsRedis;
use function outputDir;
use function sprintf;
use function uniqid;

class GetSetForeverCest
{
    /**
     * Tests Phalcon\Storage\Adapter\* :: get()/setForever()
     *
     * @dataProvider getExamples
     *
     * @author       Phalcon Team <team@phalcon.io>
     * @since        2020-09-09
     */
    public function storageAdapterGetSetForever(IntegrationTester $I, Example $example)
    {
        $I->wantToTest(
            sprintf(
                'Storage\Adapter\%s - get()/setForever()',
                $example['className']
            )
        );

        $extension = $example['extension'];
        $class     = $example['class'];
        $options   = $example['options'];

        if (!empty($extension)) {
            $I->checkExtensionIsLoaded($extension);
        }

        $serializer = new SerializerFactory();
        $adapter    = new $class($serializer, $options);

        $key = uniqid();

        $result = $adapter->setForever($key, "test");
        $I->assertTrue($result);

        sleep(2);
        $result = $adapter->has($key);
        $I->assertTrue($result);

        /**
         * Delete it
         */
        $result = $adapter->delete($key);
        $I->assertTrue($result);
    }

    /**
     * Tests Phalcon\Storage\Adapter\Weak :: get()setForever()
     *
     *
     * @author       Phalcon Team <team@phalcon.io>
     * @since        2023-07-17
     */
    public function storageAdapterWeakGetSetForever(IntegrationTester $I)
    {
        $I->wantToTest('Storage\Adapter\Weak - get()/setForever()');


        $serializer = new SerializerFactory();
        $adapter    = new Weak($serializer);

        $key = uniqid();
        $obj = new \stdClass();
        $result = $adapter->setForever($key, "test");
        $I->assertFalse($result);
        $result = $adapter->setForever($key, $obj);
        $I->assertTrue($result);
        sleep(2);
        $result = $adapter->has($key);
        $I->assertTrue($result);
        /**
         * Delete it
         */
        $result = $adapter->delete($key);
        $I->assertTrue($result);
    }

    /**
     * @return array[]
     */
    private function getExamples(): array
    {
        return [
            [
                'className' => 'Apcu',
                'class'     => Apcu::class,
                'options'   => [],
                'expected'  => null,
                'extension' => 'apcu',
            ],
            [
                'className' => 'Libmemcached',
                'class'     => Libmemcached::class,
                'options'   => getOptionsLibmemcached(),
                'expected'  => NativeMemcached::class,
                'extension' => 'memcached',
            ],
            [
                'className' => 'Memory',
                'label'     => 'default',
                'class'     => Memory::class,
                'options'   => [],
                'expected'  => null,
                'extension' => '',
            ],
            [
                'className' => 'Redis',
                'label'     => 'default',
                'class'     => Redis::class,
                'options'   => getOptionsRedis(),
                'expected'  => NativeRedis::class,
                'extension' => 'redis',
            ],
            [
                'className' => 'Stream',
                'label'     => 'default',
                'class'     => Stream::class,
                'options'   => [
                    'storageDir' => outputDir(),
                ],
                'expected'  => null,
                'extension' => '',
            ],
        ];
    }
}
